#include <iostream>
#include <vector>
#include <algorithm>

struct Participant {
    int id;
    std::string gender;
    int grade;
    int age;
    int score;
};

bool compareParticipants(const Participant& p1, const Participant& p2) {
    return p1.score > p2.score || (p1.score == p2.score && p1.id < p2.id);
}

int main() {
    int n;
    std::cin >> n;

    std::vector<Participant> participants;
    for (int i = 0; i < n; ++i) {
        Participant p;
        std::cin >> p.id >> p.gender >> p.grade >> p.age >> p.score;
        participants.push_back(p);
    }

    int m;
    std::cin >> m;

    for (int i = 0; i < m; ++i) {
        std::string olympiad;
        std::cin >> olympiad;

        std::vector<Participant> selectedParticipants;

        if (olympiad == "IOI" || olympiad == "CEOI") {
            // Select top 4 participants based on score
            std::partial_sort(participants.begin(), participants.begin() + 4, participants.end(), compareParticipants);
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + 4);
        } else if (olympiad == "EGOI") {
            // Select top 4 female participants based on score
            std::partial_sort(participants.begin(), participants.begin() + 4, participants.end(), compareParticipants);
            auto femaleParticipants = std::count_if(participants.begin(), participants.end(), [](const Participant& p) {
                return p.gender == "female";
            });
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + std::min(4, static_cast<int>(femaleParticipants)));
        } else if (olympiad == "EJOI") {
            // Select top 4 participants under 15 years old based on score
            std::partial_sort(participants.begin(), participants.begin() + 4, participants.end(), compareParticipants);
            auto youngParticipants = std::count_if(participants.begin(), participants.end(), [](const Participant& p) {
                return p.age < 15;
            });
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + std::min(4, static_cast<int>(youngParticipants)));
        } else if (olympiad == "BaltOI") {
            // Select top 6 participants based on score
            std::partial_sort(participants.begin(), participants.begin() + 6, participants.end(), compareParticipants);
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + 6);
        } else if (olympiad == "BalkOI") {
            // Select top 4 participants not in 11th grade based on score
            std::partial_sort(participants.begin(), participants.begin() + 4, participants.end(), compareParticipants);
            auto non11thGradeParticipants = std::count_if(participants.begin(), participants.end(), [](const Participant& p) {
                return p.grade != 11;
            });
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + std::min(4, static_cast<int>(non11thGradeParticipants)));
        } else if (olympiad == "JBOI") {
            // Select top 4 participants not in 11th or 10th grade based on score
            std::partial_sort(participants.begin(), participants.begin() + 4, participants.end(), compareParticipants);
            auto non11thAnd10thGradeParticipants = std::count_if(participants.begin(), participants.end(), [](const Participant& p) {
                return p.grade != 11 && p.grade != 10;
            });
            selectedParticipants.insert(selectedParticipants.end(), participants.begin(), participants.begin() + std::min(4, static_cast<int>(non11thAnd10thGradeParticipants)));
        }

        // Sort selected participants by id
        std::sort(selectedParticipants.begin(), selectedParticipants.end(), [](const Participant& p1, const Participant& p2) {
            return p1.id < p2.id;
        });

        // Output results
        std::cout << olympiad << std::endl;
        for (const auto& p : selectedParticipants) {
            std::cout << p.id << std::endl;
        }
    }

    return 0;
}
