#include <iostream>
#include <vector>
#include <algorithm>
#include <map>

struct Participant {
    int id;
    std::string gender;
    int grade;
    int age;
    int score;
};

bool compareParticipants(const Participant& a, const Participant& b) {
    return a.score > b.score || (a.score == b.score && a.id < b.id);
}

std::vector<Participant> findTeamMembers(const std::string& olympiad, const std::vector<Participant>& participants) {
    if (olympiad == "IOI" || olympiad == "CEOI") {
        std::vector<Participant> sortedParticipants = participants;
        std::sort(sortedParticipants.begin(), sortedParticipants.end(), compareParticipants);
        return std::vector<Participant>(sortedParticipants.begin(), sortedParticipants.begin() + std::min(4, static_cast<int>(sortedParticipants.size())));
    } else if (olympiad == "EGOI") {
        std::vector<Participant> females;
        for (const auto& p : participants) {
            if (p.gender == "female") {
                females.push_back(p);
            }
        }
        std::sort(females.begin(), females.end(), compareParticipants);
        return std::vector<Participant>(females.begin(), females.begin() + std::min(4, static_cast<int>(females.size())));
    } else if (olympiad == "EJOI") {
        std::vector<Participant> juniors;
        for (const auto& p : participants) {
            if (p.age <= 15) {
                juniors.push_back(p);
            }
        }
        std::sort(juniors.begin(), juniors.end(), compareParticipants);
        return std::vector<Participant>(juniors.begin(), juniors.begin() + std::min(4, static_cast<int>(juniors.size())));
    } else if (olympiad == "BaltOI") {
        std::vector<Participant> sortedParticipants = participants;
        std::sort(sortedParticipants.begin(), sortedParticipants.end(), compareParticipants);
        return std::vector<Participant>(sortedParticipants.begin(), sortedParticipants.begin() + std::min(6, static_cast<int>(sortedParticipants.size())));
    } else if (olympiad == "BalkOI") {
        std::vector<Participant> non11Graders;
        for (const auto& p : participants) {
            if (p.grade != 11) {
                non11Graders.push_back(p);
            }
        }
        std::sort(non11Graders.begin(), non11Graders.end(), compareParticipants);
        return std::vector<Participant>(non11Graders.begin(), non11Graders.begin() + std::min(4, static_cast<int>(non11Graders.size())));
    } else if (olympiad == "JBOI") {
        std::vector<Participant> non1110Graders;
        for (const auto& p : participants) {
            if (p.grade != 11 && p.grade != 10) {
                non1110Graders.push_back(p);
            }
        }
        std::sort(non1110Graders.begin(), non1110Graders.end(), compareParticipants);
        return std::vector<Participant>(non1110Graders.begin(), non1110Graders.begin() + std::min(4, static_cast<int>(non1110Graders.size())));
    }
    return std::vector<Participant>();
}

int main() {
    int n;
    std::cin >> n;

    std::vector<Participant> participants;
    for (int i = 0; i < n; ++i) {
        Participant p;
        std::cin >> p.id >> p.gender >> p.grade >> p.age >> p.score;
        participants.push_back(p);
    }

    int m;
    std::cin >> m;

    std::vector<std::string> requestedOlympiads(m);
    for (int i = 0; i < m; ++i) {
        std::cin >> requestedOlympiads[i];
    }

    std::map<std::string, std::vector<int>> participantsByOlympiad;

    for (const auto& olympiad : requestedOlympiads) {
        std::vector<Participant> teamMembers = findTeamMembers(olympiad, participants);
        for (const auto& member : teamMembers) {
            participantsByOlympiad[olympiad].push_back(member.id);
        }
    }

    for (const auto& olympiad : requestedOlympiads) {
        std::cout << olympiad << std::endl;
        std::sort(participantsByOlympiad[olympiad].begin(), participantsByOlympiad[olympiad].end());
        for (const auto& memberId : participantsByOlympiad[olympiad]) {
            std::cout << memberId << std::endl;
        }
    }

    return 0;
}
