using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
    static void Main()
    {
        // Read input values
        int n = int.Parse(Console.ReadLine());
        List<Participant> participants = new List<Participant>();

        for (int i = 0; i < n; i++)
        {
            string[] input = Console.ReadLine().Split();
            int id = int.Parse(input[0]);
            string gender = input[1];
            int grade = int.Parse(input[2]);
            int age = int.Parse(input[3]);
            int score = int.Parse(input[4]);

            participants.Add(new Participant(id, gender, grade, age, score));
        }

        int m = int.Parse(Console.ReadLine());

        // Process each olympiad
        for (int i = 0; i < m; i++)
        {
            string olympiad = Console.ReadLine();

            // Select participants based on the olympiad's criteria
            List<int> selectedIds = SelectParticipants(participants, olympiad);

            // Output the result
            Console.WriteLine(olympiad);
            Console.WriteLine(string.Join(" ", selectedIds.OrderBy(id => id)));
        }
    }

    static List<int> SelectParticipants(List<Participant> participants, string olympiad)
    {
        // Implement the selection algorithm based on the specified criteria for each olympiad
        switch (olympiad)
        {
            case "IOI":
            case "CEOI":
                return participants.OrderByDescending(p => p.Score).Take(4).Select(p => p.Id).ToList();
            case "EGOI":
                return participants.Where(p => p.Gender == "female").OrderByDescending(p => p.Score).Take(4).Select(p => p.Id).ToList();
            case "EJOI":
                return participants.Where(p => p.Age <= 15).OrderByDescending(p => p.Score).Take(4).Select(p => p.Id).ToList();
            case "BaltOI":
                return participants.OrderByDescending(p => p.Score).Take(6).Select(p => p.Id).ToList();
            case "BalkOI":
                return participants.Where(p => p.Grade != 11).OrderByDescending(p => p.Score).Take(4).Select(p => p.Id).ToList();
            case "JBOI":
                return participants.Where(p => p.Grade != 11 && p.Grade != 10).OrderByDescending(p => p.Score).Take(4).Select(p => p.Id).ToList();
            default:
                return new List<int>();
        }
    }
}

class Participant
{
    public int Id { get; }
    public string Gender { get; }
    public int Grade { get; }
    public int Age { get; }
    public int Score { get; }

    public Participant(int id, string gender, int grade, int age, int score)
    {
        Id = id;
        Gender = gender;
        Grade = grade;
        Age = age;
        Score = score;
    }
}
