#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

// Struct to represent a participant
struct Participant {
    int id;
    int score;
    int age;
    string gender;
};

// Function to determine the team for an olympiad based on the criteria
vector<Participant> determineTeam(vector<Participant>& participants, int teamSize, string criteria) {
    // Sort participants based on the total scores
    sort(participants.begin(), participants.end(), [](const Participant& a, const Participant& b) {
        return a.score > b.score;
    });

    // Filter participants based on the given criteria
    vector<Participant> team;
    if (criteria == "IOI" || criteria == "CEOI" || criteria == "BaltOI") {
        team.assign(participants.begin(), participants.begin() + min(teamSize, static_cast<int>(participants.size())));
    } else if (criteria == "EGOI") {
        team.reserve(teamSize);
        for (const auto& participant : participants) {
            if (participant.gender == "female") {
                team.push_back(participant);
            }
            if (team.size() == teamSize) {
                break;
            }
        }
    } else if (criteria == "EJOI") {
        team.reserve(teamSize);
        for (const auto& participant : participants) {
            if (participant.age <= 15) {
                team.push_back(participant);
            }
            if (team.size() == teamSize) {
                break;
            }
        }
    } else if (criteria == "BalkOI") {
        team.reserve(teamSize);
        for (const auto& participant : participants) {
            if (participant.age != 11) {
                team.push_back(participant);
            }
            if (team.size() == teamSize) {
                break;
            }
        }
    } else if (criteria == "JBOI") {
        team.reserve(teamSize);
        for (const auto& participant : participants) {
            if (participant.age != 11 && participant.age != 10) {
                team.push_back(participant);
            }
            if (team.size() == teamSize) {
                break;
            }
        }
    }

    return team;
}

int main() {
    int numParticipants;
    cin >> numParticipants;

    vector<Participant> participants(numParticipants);

    // Input participants' information
    for (int i = 0; i < numParticipants; ++i) {
        cin >> participants[i].id >> participants[i].score >> participants[i].age >> participants[i].gender;
    }

    // Determine teams for each olympiad
    vector<Participant> ioiTeam = determineTeam(participants, 4, "IOI");
    vector<Participant> ceoiTeam = determineTeam(participants, 4, "CEOI");
    vector<Participant> egoiTeam = determineTeam(participants, 4, "EGOI");
    vector<Participant> ejoiteam = determineTeam(participants, 4, "EJOI");
    vector<Participant> baltoiTeam = determineTeam(participants, 6, "BaltOI");
    vector<Participant> balkoiTeam = determineTeam(participants, 4, "BalkOI");
    vector<Participant> jboiTeam = determineTeam(participants, 4, "JBOI");

    // Output teams
    cout << "IOI" << endl;
    for (const auto& participant : ioiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "CEOI" << endl;
    for (const auto& participant : ceoiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "EGOI" << endl;
    for (const auto& participant : egoiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "EJOI" << endl;
    for (const auto& participant : ejoiteam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "BaltOI" << endl;
    for (const auto& participant : baltoiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "BalkOI" << endl;
    for (const auto& participant : balkoiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    cout << "JBOI" << endl;
    for (const auto& participant : jboiTeam) {
        cout << participant.id << " ";
    }
    cout << endl;

    return 0;
}
