#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <set>

using namespace std;

struct Participant {
    long long id;
    string gender;
    int grade;
    int age;
    int score;
};

bool compareParticipants(const Participant &p1, const Participant &p2) {
    return p1.score > p2.score || (p1.score == p2.score && p1.id < p2.id);
}

vector<long long> getTeam(const vector<Participant> &participants, int teamSize, const string &olympiad) {
    vector<long long> teamIds;
    if (olympiad == "IOI" || olympiad == "CEOI") {
        for (int i = 0; i < min(teamSize, (int)participants.size()); ++i) {
            teamIds.push_back(participants[i].id);
        }
    } else if (olympiad == "EGOI") {
        int count = 0;
        for (const auto &participant : participants) {
            if (participant.gender == "female") {
                teamIds.push_back(participant.id);
                ++count;
                if (count == teamSize) {
                    break;
                }
            }
        }
    } else if (olympiad == "EJOI") {
        int count = 0;
        for (const auto &participant : participants) {
            if (participant.age <= 15) {
                teamIds.push_back(participant.id);
                ++count;
                if (count == teamSize) {
                    break;
                }
            }
        }
    } else if (olympiad == "BaltOI") {
        for (int i = 0; i < min(teamSize, (int)participants.size()); ++i) {
            teamIds.push_back(participants[i].id);
        }
    } else if (olympiad == "BalkOI") {
        int count = 0;
        for (const auto &participant : participants) {
            if (participant.grade != 11) {
                teamIds.push_back(participant.id);
                ++count;
                if (count == teamSize) {
                    break;
                }
            }
        }
    } else if (olympiad == "JBOI") {
        int count = 0;
        for (const auto &participant : participants) {
            if (participant.grade != 11 && participant.grade != 10) {
                teamIds.push_back(participant.id);
                ++count;
                if (count == teamSize) {
                    break;
                }
            }
        }
    }
    return teamIds;
}

int main() {
    int n;
    cin >> n;

    vector<Participant> participants(n);
    for (int i = 0; i < n; ++i) {
        cin >> participants[i].id >> participants[i].gender >> participants[i].grade >> participants[i].age >> participants[i].score;
    }

    sort(participants.begin(), participants.end(), compareParticipants);

    int m;
    cin >> m;

    vector<string> olympiads(m);
    for (int i = 0; i < m; ++i) {
        cin >> olympiads[i];
    }

    map<string, vector<long long>> teamMap;

    for (const auto &olympiad : olympiads) {
        vector<long long> team = getTeam(participants, (olympiad == "BaltOI") ? 6 : 4, olympiad);
        teamMap[olympiad] = team;
    }

    for (const auto &olympiad : olympiads) {
        cout << olympiad << endl;
        for (long long id : teamMap[olympiad]) {
            cout << id << endl;
        }
    }

    return 0;
}
