using System;
using System.Collections.Generic;
using System.Linq;

class Program
{
    static void Main()
    {
        int n = int.Parse(Console.ReadLine());
        List<Participant> participants = new List<Participant>();

        for (int i = 0; i < n; i++)
        {
            string[] input = Console.ReadLine().Split();
            int id = int.Parse(input[0]);
            string gender = input[1];
            int grade = int.Parse(input[2]);
            int age = int.Parse(input[3]);
            int score = int.Parse(input[4]);

            participants.Add(new Participant(id, gender, grade, age, score));
        }

        int m = int.Parse(Console.ReadLine());
        List<string> olympiads = new List<string>();

        for (int i = 0; i < m; i++)
        {
            olympiads.Add(Console.ReadLine());
        }

        Dictionary<string, List<int>> teams = new Dictionary<string, List<int>>();

        foreach (string olympiad in olympiads)
        {
            teams.Add(olympiad, GetTeamIds(olympiad, participants));
        }

        foreach (var team in teams)
        {
            Console.WriteLine(team.Key);
            Console.WriteLine(string.Join(" ", team.Value.OrderBy(x => x)));
        }
    }

    static List<int> GetTeamIds(string olympiad, List<Participant> participants)
    {
        List<int> teamIds = new List<int>();

        switch (olympiad)
        {
            case "IOI":
            case "CEOI":
                teamIds = participants
                    .OrderByDescending(p => p.Score)
                    .Take(4)
                    .Select(p => p.Id)
                    .ToList();
                break;

            case "EGOI":
                teamIds = participants
                    .Where(p => p.Gender == "female")
                    .OrderByDescending(p => p.Score)
                    .Take(4)
                    .Select(p => p.Id)
                    .ToList();
                break;

            case "EJOI":
                teamIds = participants
                    .Where(p => p.Age <= 15)
                    .OrderByDescending(p => p.Score)
                    .Take(4)
                    .Select(p => p.Id)
                    .ToList();
                break;

            case "BaltOI":
                teamIds = participants
                    .OrderByDescending(p => p.Score)
                    .Take(6)
                    .Select(p => p.Id)
                    .ToList();
                break;

            case "BalkOI":
                teamIds = participants
                    .Where(p => p.Grade != 11)
                    .OrderByDescending(p => p.Score)
                    .Take(4)
                    .Select(p => p.Id)
                    .ToList();
                break;

            case "JBOI":
                teamIds = participants
                    .Where(p => p.Grade != 11 && p.Grade != 10)
                    .OrderByDescending(p => p.Score)
                    .Take(4)
                    .Select(p => p.Id)
                    .ToList();
                break;
        }

        return teamIds;
    }
}

class Participant
{
    public int Id { get; }
    public string Gender { get; }
    public int Grade { get; }
    public int Age { get; }
    public int Score { get; }

    public Participant(int id, string gender, int grade, int age, int score)
    {
        Id = id;
        Gender = gender;
        Grade = grade;
        Age = age;
        Score = score;
    }
}