import java.util.*;

class OlympiadParticipant {
    int id;
    String gender;
    int grade;
    int age;
    int score;

    public OlympiadParticipant(int id, String gender, int grade, int age, int score) {
        this.id = id;
        this.gender = gender;
        this.grade = grade;
        this.age = age;
        this.score = score;
    }
}

public class Main {
    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);

        // Зчитуємо кількість учасників
        int n = scanner.nextInt();
        List<OlympiadParticipant> participants = new ArrayList<>();

        // Зчитуємо інформацію про учасників
        for (int i = 0; i < n; i++) {
            int id = scanner.nextInt();
            String gender = scanner.next();
            int grade = scanner.nextInt();
            int age = scanner.nextInt();
            int score = scanner.nextInt();

            participants.add(new OlympiadParticipant(id, gender, grade, age, score));
        }

        // Зчитуємо кількість олімпіад
        int m = scanner.nextInt();
        Set<String> olympiadsToCheck = new HashSet<>();

        // Зчитуємо назви олімпіад для перевірки
        for (int i = 0; i < m; i++) {
            olympiadsToCheck.add(scanner.next());
        }

        // Сортуємо учасників за балами у зворотньому порядку (від найкращих)
        participants.sort(Comparator.comparingInt((OlympiadParticipant p) -> p.score).reversed());

        // Визначаємо склад команд для кожної олімпіади
        for (String olympiad : olympiadsToCheck) {
            List<Integer> teamIds = getTeamMembers(participants, olympiad);
            System.out.println(olympiad);
            for (int id : teamIds) {
                System.out.println(id);
            }
        }
    }

    // Метод для визначення складу команди для конкретної олімпіади
    private static List<Integer> getTeamMembers(List<OlympiadParticipant> participants, String olympiad) {
        List<Integer> teamIds = new ArrayList<>();
        int teamSize = getTeamSize(olympiad);

        String genderCriteria = getGenderCriteria(olympiad);
        int ageLimit = getAgeLimit(olympiad);

        int count = 0;
        for (OlympiadParticipant participant : participants) {
            if (!teamIds.contains(participant.id) &&
                    participant.gender.equals(genderCriteria) &&
                    participant.age <= ageLimit) {
                teamIds.add(participant.id);
                count++;

                if (count == teamSize) {
                    break;
                }
            }
        }

        return teamIds;
    }

    // Метод для визначення розміру команди для конкретної олімпіади
    private static int getTeamSize(String olympiad) {
        if (olympiad.equals("IOI") || olympiad.equals("CEOI")) {
            return 4;
        } else if (olympiad.equals("EGOI") || olympiad.equals("EJOI")) {
            return 4;
        } else if (olympiad.equals("BaltOI")) {
            return 6;
        } else if (olympiad.equals("BalkOI") || olympiad.equals("JBOI")) {
            return 4;
        }
        return 0;
    }

    // Метод для визначення критеріїв за статтю для конкретної олімпіади
    private static String getGenderCriteria(String olympiad) {
        if (olympiad.equals("EGOI")) {
            return "female";
        } else {
            return "male";
        }
    }

    // Метод для визначення обмеження за віком для конкретної олімпіади
    private static int getAgeLimit(String olympiad) {
        if (olympiad.equals("EJOI")) {
            return 15;
        } else if (olympiad.equals("BalkOI") || olympiad.equals("JBOI")) {
            return 10;
        } else {
            return 20;
        }
    }
}
