# Define a function to find the longest sequence of unmarked colors
def longest_sequence(wheel):
  # Initialize the maximum length and the start and end indices of the longest sequence
  max_length = 0
  max_start = -1
  max_end = -1
  start = -1
  end = -1
  # Loop through the wheel
  for i in range(len(wheel)):
    # If the color is unmarked, extend the current sequence
    if wheel[i] == 0:
      # If this is the first unmarked color, set the start index
      if start == -1:
        start = i
      # Update the end index
      end = i
    # If the color is marked or we reach the end of the wheel, check the current sequence
    else:
      # If the current sequence is longer than the maximum, update the maximum length and indices
      if end - start + 1 > max_length:
        max_length = end - start + 1
        max_start = start
        max_end = end
      # Reset the start and end indices
      start = -1
      end = -1
  # Check the last sequence if it wraps around the wheel
  if start != -1 and end == len(wheel) - 1:
    # If the first color is also unmarked, extend the sequence
    if wheel[0] == 0:
      end = 0
      # Loop through the wheel until we find a marked color
      while wheel[end] == 0:
        end += 1
      # Adjust the end index
      end -= 1
    # If the current sequence is longer than the maximum, update the maximum length and indices
    if end - start + 1 > max_length:
      max_length = end - start + 1
      max_start = start
      max_end = end
  # Return the maximum length and the start and end indices of the longest sequence
  return max_length, max_start, max_end

# Define a function to choose a color from the middle of a sequence
def choose_color(wheel, start, end):
  # Find the middle index of the sequence
  mid = (start + end) // 2
  # Mark the color as used
  wheel[mid] = 1
  # Return the color index
  return mid

# Read the input
n, m = map(int, input().split())
a = list(map(int, input().split()))

# Initialize the list of color wheels
wheels = []
# Loop through each wheel
for i in range(n):
  # Create a wheel with a[i] + 1 colors, all unmarked (0)
  wheel = [0] * (a[i] + 1)
  # Mark the first color as used (1)
  wheel[0] = 1
  # Add the wheel to the list
  wheels.append(wheel)

# Initialize the list of chosen colors
colors = []
# Loop m times
for i in range(m):
  # Initialize the maximum length and the index of the wheel with the longest sequence
  max_length = 0
  max_index = -1
  # Loop through each wheel
  for j in range(n):
    # Find the longest sequence of unmarked colors on the wheel
    length, start, end = longest_sequence(wheels[j])
    # If the length is greater than the maximum, update the maximum length and index
    if length > max_length:
      max_length = length
      max_index = j
      max_start = start
      max_end = end
  # Choose a color from the middle of the longest sequence on the wheel with the maximum index
  color = choose_color(wheels[max_index], max_start, max_end)
  # Add the color index and the wheel index to the list of chosen colors
  colors.append((color, max_index))

# Print the maximum length of the sequence of unmarked colors before Miss M selects the (m+1)-th color
print(max_length)
print(colors)