from collections import defaultdict

def process_gifts(colors, sizes, inventory, winners):
    remaining_inventory = {color: {size: count for size, count in sizes.items()} for color, sizes in inventory.items()}
    ordered_gifts = defaultdict(list)
    unordered_gifts = defaultdict(list)

    for winner in winners:
        color, size = int(winner[0]), winner[1]
        if color in remaining_inventory and size in remaining_inventory[color] and remaining_inventory[color][size] > 0:
            remaining_inventory[color][size] -= 1
            ordered_gifts[color].append(size)
        else:
            alternatives = [alt_size for alt_size, alt_count in remaining_inventory[color].items() if alt_count > 0]
            if alternatives:
                alternatives.sort()
                alt_size = alternatives[0]
                remaining_inventory[color][alt_size] -= 1
                unordered_gifts[color].append(alt_size)
            else:
                for alt_color, alt_sizes in remaining_inventory.items():
                    available_sizes = [alt_size for alt_size, alt_count in alt_sizes.items() if alt_count > 0]
                    if available_sizes:
                        available_sizes.sort()
                        alt_size = available_sizes[0]
                        remaining_inventory[alt_color][alt_size] -= 1
                        unordered_gifts[alt_color].append(alt_size)
                        break

    return remaining_inventory, ordered_gifts, unordered_gifts

n, m = map(int, input().split())
sizes = input().split()
inventory = {i: {size: int(count) for size, count in enumerate(input().split(), 1)} for i in range(1, n + 1)}
k = int(input())
winners = [list(map(str, input().split())) for _ in range(k)]

remaining_inventory, ordered_gifts, unordered_gifts = process_gifts(range(1, n + 1), sizes, inventory, winners)

for i in range(1, n + 1):
    print(*[remaining_inventory[i][j] for j in range(1, m + 1)])

for i in range(1, n + 1):
    print(*[unordered_gifts[i][j] for j in range(1, m + 1)])