using System;
using System.Collections.Generic;

public class TShirt
{
    public int Color { get; set; }
    public int Size { get; set; }

    public TShirt(int color, int size)
    {
        Color = color;
        Size = size;
    }
}

public class Program
{
    public static void Main(string[] args)
    {
        // Read input
        string[] nm = Console.ReadLine().Split();
        int n = int.Parse(nm[0]);
        int m = int.Parse(nm[1]);

        string[] sizes = Console.ReadLine().Split();

        int[,] availableTShirts = new int[n, m];
        for (int i = 0; i < n; i++)
        {
            string[] counts = Console.ReadLine().Split();
            for (int j = 0; j < m; j++)
            {
                availableTShirts[i, j] = int.Parse(counts[j]);
            }
        }

        int k = int.Parse(Console.ReadLine());

        List<TShirt> winnersRequests = new List<TShirt>();
        for (int i = 0; i < k; i++)
        {
            string[] request = Console.ReadLine().Split();
            int color = int.Parse(request[0]);
            int size = Array.IndexOf(sizes, request[1]);
            winnersRequests.Add(new TShirt(color, size));
        }

        // Process requests
        int[,] remainingTShirts = new int[n, m];
        int[,] neededTShirts = new int[n, m];

        for (int i = 0; i < k; i++)
        {
            TShirt request = winnersRequests[i];
            bool foundTShirt = false;

            // Check if requested T-shirt is available
            if (availableTShirts[request.Color, request.Size] > 0)
            {
                availableTShirts[request.Color, request.Size]--;
                foundTShirt = true;
            }
            else
            {
                // Look for T-shirts of the same size but different color
                for (int j = 0; j < n; j++)
                {
                    if (j != request.Color && availableTShirts[j, request.Size] > 0)
                    {
                        availableTShirts[j, request.Size]--;
                        foundTShirt = true;
                        break;
                    }
                }
            }

            // Reduce size and color if no suitable T-shirt is found
            if (!foundTShirt)
            {
                for (int s = request.Size + 1; s < m; s++)
                {
                    for (int c = 0; c < n; c++)
                    {
                        if (c == request.Color && availableTShirts[c, s] > 0)
                        {
                            availableTShirts[c, s]--;
                            foundTShirt = true;
                            break;
                        }
                        else if (availableTShirts[c, s] > 0)
                        {
                            availableTShirts[c, s]--;
                            foundTShirt = true;
                            break;
                        }
                    }
                    if (foundTShirt)
                        break;
                }
            }

            // Record remaining and needed T-shirts
            if (foundTShirt)
            {
                remainingTShirts[request.Color, request.Size]++;
            }
            else
            {
                neededTShirts[request.Color, request.Size]++;
            }
        }

        // Output remaining and needed T-shirts
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < m; j++)
            {
                Console.Write(remainingTShirts[i, j] + " ");
            }
            Console.WriteLine();
        }

        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < m; j++)
            {
                Console.Write(neededTShirts[i, j] + " ");
            }
            Console.WriteLine();
        }
    }
}
