#include <iostream>
#include <vector>
#include <algorithm>
#include <unordered_map>

struct Participant {
    int id;
    std::string gender;
    int grade;
    int age;
    int score;
};

std::vector<Participant> find_team_members(std::string olympiad, std::vector<Participant>& participants) {
    if (olympiad == "IOI" || olympiad == "CEOI") {
        std::sort(participants.begin(), participants.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(participants.begin(), participants.begin() + 4);
    } else if (olympiad == "EGOI") {
        std::vector<Participant> females;
        for (const auto& p : participants) {
            if (p.gender == "female") {
                females.push_back(p);
            }
        }
        std::sort(females.begin(), females.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(females.begin(), females.begin() + std::min(4, static_cast<int>(females.size())));
    } else if (olympiad == "EJOI") {
        std::vector<Participant> juniors;
        for (const auto& p : participants) {
            if (p.age <= 15) {
                juniors.push_back(p);
            }
        }
        std::sort(juniors.begin(), juniors.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(juniors.begin(), juniors.begin() + std::min(4, static_cast<int>(juniors.size())));
    } else if (olympiad == "BaltOI") {
        std::sort(participants.begin(), participants.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(participants.begin(), participants.begin() + 6);
    } else if (olympiad == "BalkOI") {
        std::vector<Participant> non_11_graders;
        for (const auto& p : participants) {
            if (p.grade != 11) {
                non_11_graders.push_back(p);
            }
        }
        std::sort(non_11_graders.begin(), non_11_graders.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(non_11_graders.begin(), non_11_graders.begin() + std::min(4, static_cast<int>(non_11_graders.size())));
    } else if (olympiad == "JBOI") {
        std::vector<Participant> non_11_10_graders;
        for (const auto& p : participants) {
            if (p.grade != 11 && p.grade != 10) {
                non_11_10_graders.push_back(p);
            }
        }
        std::sort(non_11_10_graders.begin(), non_11_10_graders.end(), [](const Participant& a, const Participant& b) {
            return a.score != b.score ? a.score > b.score : a.id < b.id;
        });
        return std::vector<Participant>(non_11_10_graders.begin(), non_11_10_graders.begin() + std::min(4, static_cast<int>(non_11_10_graders.size())));
    }

    return std::vector<Participant>();
}

int main() {
    int n, m;
    std::cin >> n;

    std::vector<Participant> participants;
    for (int i = 0; i < n; ++i) {
        Participant p;
        std::cin >> p.id >> p.gender >> p.grade >> p.age >> p.score;
        participants.push_back(p);
    }

    std::cin >> m;
    std::vector<std::string> requested_olympiads(m);
    for (int i = 0; i < m; ++i) {
        std::cin >> requested_olympiads[i];
    }

    std::unordered_map<std::string, std::vector<int>> participants_by_olympiad;

    for (const auto& olympiad : requested_olympiads) {
        auto team_members = find_team_members(olympiad, participants);
        for (const auto& member : team_members) {
            participants_by_olympiad[olympiad].push_back(member.id);
        }
    }

    for (const auto& olympiad : requested_olympiads) {
        std::cout << olympiad << std::endl;
        auto& member_ids = participants_by_olympiad[olympiad];
        std::sort(member_ids.begin(), member_ids.end());
        for (const auto& member_id : member_ids) {
            std::cout << member_id << std::endl;
        }
    }

    return 0;
}
