#include <iostream>
#include <vector>
#include <map>
#include <algorithm>

struct Competitor {
    int id, grade, age, score;
    std::string gender;
};

std::vector<int> getIOITeam(const std::vector<Competitor>& competitors) {
    std::vector<int> ioiTeam;
    std::vector<Competitor> filtered = competitors;
    std::sort(filtered.begin(), filtered.end(), [](const Competitor& a, const Competitor& b) {
        return (a.score > b.score || (a.score == b.score && a.id < b.id));
    });

    for (int i = 0; i < 4 && i < filtered.size(); ++i) {
        ioiTeam.push_back(filtered[i].id);
    }
    return ioiTeam;
}

std::vector<int> getCEOITeam(const std::vector<Competitor>& competitors) {
    std::vector<int> ceoiTeam;
    std::vector<Competitor> filtered = competitors;
    std::sort(filtered.begin(), filtered.end(), [](const Competitor& a, const Competitor& b) {
        return (a.score > b.score || (a.score == b.score && a.id < b.id));
    });

    for (int i = 0; i < 4 && i < filtered.size(); ++i) {
        ceoiTeam.push_back(filtered[i].id);
    }
    return ceoiTeam;
}

std::vector<int> getEGOITeam(const std::vector<Competitor>& competitors) {
    std::vector<int> egoiTeam;
    std::vector<Competitor> filtered = competitors;
    std::sort(filtered.begin(), filtered.end(), [](const Competitor& a, const Competitor& b) {
        return ((a.gender == "female" && a.score > b.score) || (a.gender == "female" && a.score == b.score && a.id < b.id));
    });

    for (const auto& competitor : filtered) {
        if (competitor.gender == "female" && egoiTeam.size() < 4) {
            egoiTeam.push_back(competitor.id);
        }
    }
    return egoiTeam;
}

// Додайте функції для інших олімпіад
// ...

std::map<std::string, std::vector<int>> getTeams(const std::vector<Competitor>& competitors, const std::vector<std::string>& olympiads) {
    std::map<std::string, std::vector<int>> teams;

    for (const auto& olympiad : olympiads) {
        if (olympiad == "IOI") {
            teams[olympiad] = getIOITeam(competitors);
        } else if (olympiad == "CEOI") {
            teams[olympiad] = getCEOITeam(competitors);
        } else if (olympiad == "EGOI") {
            teams[olympiad] = getEGOITeam(competitors);
        }
        // Додайте умови для інших олімпіад
        // ...
    }

    return teams;
}

int main() {
    int n;
    std::cin >> n;

    std::vector<Competitor> participants(n);
    for (int i = 0; i < n; ++i) {
        std::cin >> participants[i].id >> participants[i].gender >> participants[i].grade >> participants[i].age >> participants[i].score;
    }

    int m;
    std::cin >> m;

    std::vector<std::string> olympiads(m);
    for (int i = 0; i < m; ++i) {
        std::cin >> olympiads[i];
    }

    std::map<std::string, std::vector<int>> teams = getTeams(participants, olympiads);

    for (const auto& olympiad : olympiads) {
        std::cout << olympiad << std::endl;
        for (const auto& participant : teams[olympiad]) {
            std::cout << participant << std::endl;
        }
    }

    return 0;
}
