#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

// A function to find the longest sequence of unmarked colors among all wheels
int longest_sequence(vector<vector<bool>>& wheels) {
    int longest = 0;
    // Loop through the wheels
    for (auto& wheel : wheels) {
        int current = 0;
        // Loop through the colors
        for (auto& color : wheel) {
            // If the color is unmarked, increment the current sequence length
            if (!color) {
                current++;
            }
            // If the color is marked or the end of the wheel is reached, update the longest sequence length
            else {
                longest = max(longest, current);
                current = 0;
            }
        }
        // Check the last sequence
        longest = max(longest, current);
    }
    return longest;
}

// A function to choose a color from the longest sequence of unmarked colors
void choose_color(vector<vector<bool>>& wheels) {
    int longest = longest_sequence(wheels);
    // Loop through the wheels
    for (auto& wheel : wheels) {
        int current = 0;
        int start = -1;
        // Loop through the colors
        for (int i = 0; i < wheel.size(); i++) {
            // If the color is unmarked, increment the current sequence length and update the start index
            if (!wheel[i]) {
                current++;
                if (start == -1) {
                    start = i;
                }
            }
            // If the color is marked or the end of the wheel is reached, check if the current sequence is the longest
            else {
                if (current == longest) {
                    // If the length is odd, choose the middle color and mark it as true
                    if (current % 2 == 1) {
                        wheel[start + current / 2] = true;
                        return;
                    }
                    // If the length is even, choose the left middle color and mark it as true
                    else {
                        wheel[start + current / 2 - 1] = true;
                        return;
                    }
                }
                current = 0;
                start = -1;
            }
        }
        // Check the last sequence
        if (current == longest) {
            // If the length is odd, choose the middle color and mark it as true
            if (current % 2 == 1) {
                wheel[start + current / 2] = true;
                return;
            }
            // If the length is even, choose the left middle color and mark it as true
            else {
                wheel[start + current / 2 - 1] = true;
                return;
            }
        }
    }
}

int main() {
    // Read the number of wheels and the number of colors to choose
    int n, m;
    cin >> n >> m;
    // Create a vector to store the wheels
    vector<vector<bool>> wheels(n);
    // Read the number of colors on each wheel
    for (int i = 0; i < n; i++) {
        int a;
        cin >> a;
        // Create a vector of size a + 1 and fill it with false
        vector<bool> wheel(a + 1, false);
        // Mark the first color as true
        wheel[0] = true;
        // Add the wheel to the vector of wheels
        wheels[i] = wheel;
    }
    // Choose m colors using the algorithm
    for (int i = 0; i < m; i++) {
        choose_color(wheels);
    }
    // Print the maximum length of the sequence of unmarked colors before choosing the (m + n)-th color
    cout << longest_sequence(wheels) << endl;
    return 0;
}
